use Test::Simple tests => 14;

use E2::Interface;	ok 1;
use E2::Node;		ok 1;
use E2::E2Node;		ok 1;
use E2::Writeup;	ok 1;
use E2::Superdoc;	ok 1;
use E2::Room;		ok 1;
use E2::User;		ok 1;
use E2::Usergroup;	ok 1;
use E2::Ticker;		ok 1;
use E2::Message;	ok 1;
use E2::Search;		ok 1;
use E2::UserSearch;	ok 1;
use E2::Session;	ok 1;
use E2::ClientVersion;	ok 1;
